import java.awt.*;
import java.util.*;

/**The list of actions for this element or timeline is displayed on
 * the left hand side, and can be clicked on to edit. Below is a box for
 * creating a new action (by choosing event and action type). To the right
 * is the action-specific editing panel
 */
class ActionsWindow extends FixedFrame implements CbButtonCallback
{
	Presentation pres;
	ActionsWindowCallback callback;
	Object actionsobj;
	String alist[] = { "GotoTimeLine", "GotoURL", "PlaySound", "Move",
			   "SetSize", "SetColor", "Message" };
	CbButton delete, create, ok, cancel;
	List actions;
	Choice actionch, eventch;
	Panel mid;
	ActionEditor editor;
	Vector acts = new Vector();

	/**Create a window for editing actions associated with elements
	 */
	ActionsWindow(Presentation p, ActionsWindowCallback cb, Object o)
	{
	super(450, 400);
	pres = p;
	callback = cb;
	actionsobj = o;
	acts = callback.listActions(actionsobj);
	int elist[] = callback.getEvents(actionsobj);
	setLayout(new BorderLayout());

	Panel left = new GrayPanel();
	left.setLayout(new BorderLayout());
	Panel top = new BorderPanel(1);
	top.setLayout(new BorderLayout());
	top.add("North", new Label("Existing Actions"));
	Panel top2 = new GrayPanel();
	top2.setLayout(new GridLayout(1, 1));
	top2.add(actions = new List());
	for(int i=0; i<acts.size(); i++)
		addToList((Action)acts.elementAt(i));
	top.add("Center", top2);
	Panel top3 = new GrayPanel();
	top3.setLayout(new FlowLayout(FlowLayout.RIGHT));
	top3.add(delete = new CbButton("Delete", this));
	top.add("South", top3);
	left.add("Center", top);

	Panel bot = new BorderPanel(1);
	bot.setLayout(new BorderLayout());
	bot.add("North", new Label("New Action"));
	Panel bot2 = new GrayPanel();
	bot2.setLayout(new BorderLayout());
	Panel bot21 = new GrayPanel();
	bot21.setLayout(new GridLayout(2, 1));
	bot21.add(new Label("Event:"));
	bot21.add(new Label("Action:"));
	bot2.add("West", bot21);
	Panel bot22 = new GrayPanel();
	bot22.setLayout(new GridLayout(2, 1));
	bot22.add(eventch = new Choice());
	for(int i=0; i<elist.length; i++)
		eventch.addItem(Action.ename[elist[i]]);
	bot22.add(actionch = new Choice());
	for(int i=0; i<alist.length; i++)
		actionch.addItem(alist[i]);
	bot2.add("Center", bot22);
	bot.add("Center", bot2);
	Panel bot3 = new GrayPanel();
	bot3.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot3.add(create = new CbButton("Create", this));
	bot.add("South", bot3);
	left.add("South", bot);
	add("West", left);

	mid = new BorderPanel(1);
	mid.setLayout(new GridLayout(1,1));
	add("Center", mid);

	Panel bbot = new GrayPanel();
	bbot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bbot.add(ok = new CbButton("Ok", this));
	bbot.add(cancel = new CbButton("Cancel", this));
	add("South", bbot);

	pack();
	show();
	setTitle("Edit Actions");
	Util.recursiveBackground(this, Color.lightGray);
	}

	void addToList(Action a)
	{
	actions.addItem(Action.ename[a.event]+"->"+a.getClass().getName());
	}


	public void click(CbButton b)
	{
	if (b == ok) {
		// update actions list
		callback.setActions(actionsobj, acts);
		}
	if (b == ok || b == cancel)
		dispose();
	else if (b == create) {
		// Create a new action object and start editing it
		String cn = actionch.getSelectedItem()+"Action";
		Action na = (Action)Util.createObject(cn);
		na.owner = actionsobj;
		na.event = callback.getEvents(actionsobj)
				[eventch.getSelectedIndex()];
		editAction(na);
		}
	else if (b == delete) {
		// delete the selected action
		int i = actions.getSelectedIndex();
		if (i >= 0) {
			acts.removeElementAt(i);
			actions.delItem(i);
			if (editor != null) mid.remove(editor);
			}
		}
	}

	public void dispose()
	{
	super.dispose();
	callback.actionsWindowClosed(actionsobj);
	}

	public boolean action(Event evt, Object obj)
	{
	if (evt.target == actions) {
		// action chosen from list.. display it
		int i = actions.getSelectedIndex();
		editAction((Action)acts.elementAt(i));
		return true;
		}
	return false;
	}

	/**Display the appropriate editor for this action
	 */
	void editAction(Action a)
	{
	String cn = a.getClass().getName();
	ActionEditor ed = (ActionEditor)Util.createObject(cn+"Editor");
	if (editor != null)
		mid.remove(editor);
	ed.edit(this, a);
	mid.add(ed);
	Util.recursiveLayout(mid);
	Util.recursiveBackground(mid, Color.lightGray);
	editor = ed;
	}
}

interface ActionsWindowCallback
{
	/**Called back when the actions window is closed */
	public void actionsWindowClosed(Object o);

	/**This must return a list of all the actions for the element or
	 * timeline being edited. */
	public Vector listActions(Object o);

	/**Set the list of actions for the element or timeline being edited
	 */
	public void setActions(Object o, Vector a);

	/**This must return a list of the possible events for this timeline
	 * or element. */
	public int[] getEvents(Object o);
}

