import java.awt.*;
import java.util.*;

class CurveElementEditor extends ElementEditor
{
	TextField xfac, yfac;
	CbColorButton fcol;
	CbButton points;
	Vector pts;
	CurvePointsWindow pointswin;
	CurveElement c;

	protected void makeUI()
	{
	super.makeUI();
	c = (CurveElement)elm;
	pts = c.pts;
	pan.setLayout(new GridLayout(0, 2));
	pan.add(new Label("Points:"));
	pan.add(points = new CbButton("Edit..", this));
	pan.add(new Label("X Scale:"));
	pan.add(xfac = new TextField(String.valueOf(c.xfac)));
	pan.add(new Label("Y Scale:"));
	pan.add(yfac = new TextField(String.valueOf(c.yfac)));
	pan.add(new Label("Color:"));
	pan.add(fcol = new CbColorButton(c.fcol));
	}

	protected boolean apply()
	{
	float x, y;
	try { x = Float.valueOf(xfac.getText()).floatValue(); }
	catch(NumberFormatException e) { x = -1; }
	if (x <= 0) {
		new ErrorWindow("X scale must be a positive number");
		return false;
		}
	try { y = Float.valueOf(yfac.getText()).floatValue(); }
	catch(NumberFormatException e) { y = -1; }
	if (y <= 0) {
		new ErrorWindow("Y scale must be a positive number");
		return false;
		}
	Color o = fcol.col;
	if (!super.apply()) return false;
	c.xfac = c.yfac = 1;
	c.ob = c.getBounds();
	c.xfac = x; c.yfac = y;
	c.point_c = null;
	c.resized = (x != 1 || y != 1);
	c.fcol = o;
	c.pts = pts;
	return true;
	}

	public void click(CbButton b)
	{
	super.click(b);
	if (b == points && pointswin == null) {
		new CurvePointsWindow(this);
		}
	}
}


class CurvePointsWindow extends FixedFrame implements CbButtonCallback,
                                                 CbScrollbarCallback
{
	CurveElementEditor parent;
	CbButton ok, cancel;
	Panel midc, scr;
	CbScrollbar sb;
	boolean firstpaint = true;

	CurvePointsWindow(CurveElementEditor p)
	{
	super(250, 250);
	parent = p;
	parent.pointswin = this;
	setLayout(new BorderLayout());

	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(ok = new CbButton("Ok", this));
	bot.add(cancel = new CbButton("Cancel", this));
	add("South", bot);

	Panel cen = new BorderPanel(1);
	cen.setLayout(new BorderLayout());
	cen.add("East", sb = new CbScrollbar(CbScrollbar.VERTICAL, this));
	Panel mid = new GrayPanel();
	mid.setLayout(new BorderLayout());
	Panel midt = new GrayPanel();
	midt.setLayout(new GridLayout(1, 3));
	midt.add(new Label("Point"));
	midt.add(new Label("X"));
	midt.add(new Label("Y"));
	mid.add("North", midt);
	midc = new GrayPanel();
	midc.setLayout(null);
	midc.add(scr = new GrayPanel());
	scr.setLayout(new GridLayout(0, 3));
	mid.add("Center", midc);
	cen.add("Center", mid);
	add("Center", cen);

	// create points fields
	Vector pv = parent.pts;
	for(int i=0; i<pv.size(); i++) {
		Point pt = (Point)pv.elementAt(i);
		scr.add(new Label(String.valueOf(i)));
		scr.add(new TextField(String.valueOf(pt.x)));
		scr.add(new TextField(String.valueOf(pt.y)));
		}

	// bring up window
	pack();
	show();
	setTitle("Edit Points");
	Util.recursiveBackground(this, Color.lightGray);
	}

	Vector getPoints()
	{
	Vector rv = new Vector();
	Component c[] = scr.getComponents();
	for(int i=0,j=0; i<c.length; i+=3,j++) {
		int x = parent.getNum((TextField)c[i+1]),
		    y = parent.getNum((TextField)c[i+2]);
		if (x < 0) {
			new ErrorWindow("The X value in row "+j+" mmust be "+
			                "a positive integer");
			return null;
			}
		if (y < 0) {
			new ErrorWindow("The Y value in row "+j+" mmust be "+
			                "a positive integer");
			return null;
			}
		rv.addElement(new Point(x, y));
		}
	return rv;
	}

	public boolean handleEvent(Event evt)
	{
	if (evt.id == Event.WINDOW_MOVED) {
		reshape(location().x, location().y,
			size().width, size().height);
		return true;
		}
	return super.handleEvent(evt);
	}

	public void paint(Graphics g)
	{
	if (firstpaint)
		reshape(location().x, location().y,
			size().width, size().height);
	firstpaint = false;
	}

	public void reshape(int nx, int ny, int nw, int nh)
	{
	scr.layout();
	super.reshape(nx, ny, nw, nh);
	int sh = scr.minimumSize().height;
	Dimension midcs = midc.size();
	if (midcs.height <= 0) return;
	scr.resize(midcs.width, sh);
	int v = sb.getValue(), mv = sh-midcs.height;
	sb.setValues(mv<0 ? 0 : v > mv ? mv : v, midcs.height, sh);
	}

	public void click(CbButton b)
	{
	if (b == ok) {
		Vector np = getPoints();
		if (np != null)
			parent.pts = getPoints();
		else
			return;
		}
	if (b == ok || b == cancel)
		dispose();
	}

	public void dispose()
	{
	super.dispose();
	parent.pointswin = null;
	}

	public void moved(CbScrollbar s, int p)
	{
	scr.move(0, -p);
	}

	public void moving(CbScrollbar s, int p)
	{
	}
}

