import java.awt.*;
import java.util.*;

public class EditArea extends Panel implements CbScrollbarCallback,
					       ActionsWindowCallback
{
	static final int POINT = 0;
	static final int SELECT = 1;
	static final int CREATE_RECT = 2;
	static final int CREATE_LINE = 3;
	static final int CREATE_FREE = 4;
	static final int CREATE_POLY = 5;
	static final int DEFINE_FREE = 6;
	static final int DEFINE_POLY = 7;

	jshock parent;
	CbScrollbar hsl, vsl;		// 
	int tx = 0, ty = 0;		// top-left corner position
	Presentation pres;
	Image dbuf;
	int mode = POINT;
	String create_class;
	int gridw, gridh;
	boolean gridmode;
	ActionsWindow actionswin;
	ElementChooser chooser;

	boolean dragging;
	int dragx1, dragy1, dragx2, dragy2;

	// For moving elements
	boolean moving;
	Vector selelm = new Vector();
	Vector lockelm = new Vector();
	int movex, movey, movestartx, movestarty;
	Element moveelm;
	long lastclick;

	// For resizing
	boolean resizing;
	int resizemode;
	int resizex, resizey;

	// For creating freehand elements
	boolean freehand;
	Vector freepts;

	// For creating polygon-based elements
	boolean polygon;
	Vector polypts;

	// For defining paths
	boolean defining;
	int define_start, define_end;

	EditArea(jshock p)
	{
	parent = p;
	setLayout(new BorderLayout());
	add("South", hsl = new CbScrollbar(CbScrollbar.HORIZONTAL, this));
	add("East", vsl = new CbScrollbar(CbScrollbar.VERTICAL, this));
	}

	void setPresentation(Presentation p)
	{
	pres = p;
	dbuf = Util.createImage(pres.w, pres.h);
	parent.time_line.setMaximum(pres.ctl.len);
	parent.showTimeLines();
	render();
	resize(size().width, size().height);
	}

	void createMode(int m, String s)
	{
	mode = m;
	create_class = s;
	selelm.removeAllElements();
	updateMenus();
	moving = resizing = freehand = polygon = defining = false;
	repaint();
	}

	/**In point mode, elements can be clicked on to select, double-clicked
	 * to edit and dragged to move/resize
	 */
	void pointMode()
	{
	mode = POINT;
	moving = resizing = freehand = polygon = defining = false;
	}

	/**In select mode, 
	 */
	void selectMode()
	{
	mode = SELECT;
	moving = resizing = freehand = polygon = defining = false;
	}

	void pathMode(int s, int e, int m)
	{
	mode = m; defining = true;
	define_start = s; define_end = e;
	}

	void foreChosen(Color c)
	{
	}

	void backChosen(Color c)
	{
	}

	void lock(ElementEditor ee, Element e)
	{
	e.locked = ee;
	lockelm.addElement(e);
	repaint();
	}

	void unlock(ElementEditor ee, Element e)
	{
	e.locked = null;
	lockelm.removeElement(e);
	repaint();
	}

	/**Switch to displaying the given timeline
	 */	
	void gotoTimeLine(int n)
	{
	TimeLine nt = (TimeLine)pres.tl.elementAt(n);
	pres.ctl = nt;
	selelm.removeAllElements();
	for(int i=0; i<lockelm.size(); i++) {
		Element le = (Element)lockelm.elementAt(i);
		le.locked.dispose();
		le.locked = null;
		}
	lockelm.removeAllElements();
	parent.time_line.setMaximum(pres.ctl.len);
	parent.gotoTime(pres.ctl.time);
	parent.showTimeLines();
	}

	/**Raise the currently select elements
	 */
	void raiseSelected()
	{
	int n = pres.ctl.numElements()-1;
	for(int i=0; i<selelm.size(); i++)
		pres.ctl.moveElement((Element)selelm.elementAt(i), n);
	repaint();
	}

	/**Send all selected elements to the back
	 */
	void lowerSelected()
	{
	for(int i=0; i<selelm.size(); i++)
		pres.ctl.moveElement((Element)selelm.elementAt(i), 0);
	repaint();
	}

	/**Delete all selected elements
	 */
	void deleteSelected()
	{
	for(int i=0; i<selelm.size(); i++)
		pres.ctl.removeElement((Element)selelm.elementAt(i));
	selelm.removeAllElements();
	updateMenus();
	repaint();
	}

	/**Returns and removes the selected elements
	 */
	Element[] cutSelected()
	{
	Element rv[] = new Element[selelm.size()];
	for(int i=0; i<selelm.size(); i++)
		rv[i] = (Element)selelm.elementAt(i);
	deleteSelected();
	return rv;
	}

	/**Returns a copy of all the selected elements
	 */
	Element[] copySelected()
	{
	Element rv[] = new Element[selelm.size()];
	for(int i=0; i<selelm.size(); i++)
		rv[i] = ((Element)selelm.elementAt(i)).copy();
	return rv;
	}

	/**Adds a copy of all the given elements to this timeline
	 */
	void pasteElements(Element p[])
	{
	selelm.removeAllElements();
	for(int i=0; i<p.length; i++) {
		Element ne = p[i].copy();
		while(pres.findElement(ne.name) != null)
			ne.name = "Copy_Of_"+ne.name;
		pres.ctl.addElement(ne);
		selelm.addElement(ne);
		}
	pres.ctl.setTime(pres.ctl.time);
	repaint();
	}

	void setGrid(int w, int h)
	{
	gridw = w; gridh = h;
	gridmode = (gridw != 0 || gridh != 0);
	repaint();
	}

	/**Called after change in the selection status, to enable or
	 * disable menus.
	 */
	private void updateMenus()
	{
	if (selelm.size() == 0) parent.noneSelected();
	else parent.someSelected();
	}

	/**Bring up the window for editing actions for this timeline
	 */
	void editActions()
	{
	if (actionswin == null)
		actionswin = new ActionsWindow(pres, this, pres.ctl);
	}

	/**Select all elements (except locked ones)
	 */
	void selectAll()
	{
	selelm.removeAllElements();
	parent.noneSelected();
	for(int i=0; i<pres.ctl.elmc; i++) {
		Element e = pres.ctl.elm[i];
		if (lockelm.indexOf(e) < 0)
			selelm.addElement(e);
		}
	if (selelm.size() > 0)
		parent.someSelected();
	repaint();
	}

	/**Open editing windows for the selected elements
	 */
	void editSelected()
	{
	for(int i=0; i<selelm.size(); i++)
		openEditWindow((Element)selelm.elementAt(i));
	selelm.removeAllElements();
	}

	/**Display window for selecting elements by name
	 */
	void openElementChooser()
	{
	if (chooser == null)
		new ElementChooser(this);
	}

	/**Copy the double buffer to the front.
	 */
	public void paint(Graphics g)
	{
	g.setColor(Color.lightGray);
	g.fillRect(pres.w-tx, 0, size().width, size().height);
	g.fillRect(0, pres.h-ty, size().width, size().height);
	if (dbuf == null) return;
	g.drawImage(dbuf, -tx, -ty, this);
	}

	/**Re-draw the buffer and call paint()
	 */
	public void update(Graphics g)
	{
	render();
	paint(g);
	}

	/**Draw the presentation into the double buffer
	 */
	void render()
	{
	if (dbuf == null) return;
	Graphics g = dbuf.getGraphics();

	// draw presentation, with grid
	pres.ctl.paintBackground(g, true);
	if (gridw != 0 || gridh != 0) {
		g.setColor(new Color(200, 200, 255));
		for(int x=0; x<pres.w; x+=gridw) g.drawLine(x, 0, x, pres.h);
		for(int y=0; y<pres.h; y+=gridh) g.drawLine(0, y, pres.w, y);
		}
	pres.ctl.paintElements(g, true);

	// draw drag-select or create outlines
	g.setColor(Color.black);
	g.setXORMode(Color.white);
	if (dragging && mode == CREATE_RECT) {
		g.drawRect(dragx1, dragy1, dragx2-dragx1, dragy2-dragy1);
		}
	else if (dragging && mode == CREATE_LINE) {
		g.drawLine(dragx1, dragy1, dragx2, dragy2);
		}
	else if (freehand) {
		Point prv = (Point)freepts.elementAt(0);
		for(int i=1; i<freepts.size(); i++) {
			Point nxt = (Point)freepts.elementAt(i);
			g.drawLine(prv.x, prv.y, nxt.x, nxt.y);
			prv = nxt;
			}
		}
	else if (polygon) {
		Point prv = (Point)polypts.elementAt(0);
		for(int i=1; i<polypts.size(); i++) {
			Point nxt = (Point)polypts.elementAt(i);
			g.drawLine(prv.x, prv.y, nxt.x, nxt.y);
			prv = nxt;
			}
		g.drawLine(prv.x, prv.y, dragx2, dragy2);
		}
	else if (dragging && mode == SELECT) {
		Util.dottedRect(g, dragx1, dragy1, dragx2, dragy2, 5);
		}

	// draw selected element boxes
	for(int i=0; i<selelm.size(); i++) {
		Element e = (Element)selelm.elementAt(i);
		int s = e.getStyle();
		if (s == 0) {
			Rectangle r = e.getBounds();
			Util.dottedRect(g, r.x, r.y,
			                r.x+r.width-1, r.y+r.height-1, 5);

			g.drawRect(r.x, r.y, 2, 2);
			g.drawRect(r.x+r.width-3, r.y, 2, 2);
			g.drawRect(r.x, r.y+r.height-3, 2, 2);
			g.drawRect(r.x+r.width/2-1, r.y, 2, 2);
			g.drawRect(r.x, r.y+r.height/2-1, 2, 2);

			g.fillRect(r.x+r.width-3, r.y+r.height-3, 3, 3);
			g.fillRect(r.x+r.width/2-1, r.y+r.height-3, 3, 3);
			g.fillRect(r.x+r.width-3, r.y+r.height/2-1, 3, 3);
			}
		else {
			Point ep[] = e.getEndPoints();
			g.drawRect(ep[0].x-2, ep[0].y-2, 5, 5);
			g.fillRect(ep[1].x-2, ep[1].y-2, 6, 6);
			}
		}

	// draw locked element boxes
	for(int i=0; i<lockelm.size(); i++) {
		Element e = (Element)lockelm.elementAt(i);
		int s = e.getStyle();
		Rectangle r;
		if (s == 0) r = e.getBounds();
		else {
			Point ep[] = e.getEndPoints(), p0 = ep[0], p1 = ep[1];
			int rx, ry, rw, rh;
			if (p0.x < p1.x) { rx = p0.x; rw = p1.x - p0.x; }
			else { rx = p1.x; rw = p0.x - p1.x; }
			if (p0.y < p1.y) { ry = p0.y; rh = p1.y - p0.y; }
			else { ry = p1.y; rh = p0.y - p1.y; }
			r = new Rectangle(rx, ry, rw, rh);
			}
		Util.dottedRect(g, r.x, r.y, r.x+r.width-1, r.y+r.height-1, 10);
		Util.dottedRect(g, r.x+1, r.y+1,
		                r.x+r.width-2, r.y+r.height-2, 10);
		}
	g.setPaintMode();
	}

	public boolean mouseDown(Event e, int x, int y)
	{
	x += tx; y += ty;
	int gx = x, gy = y;
	if (gridmode) {
		gx = (x/gridw)*gridw;
		gy = (y/gridh)*gridh;
		}
	if (mode == POINT) {
		// select clicked-on item, or start dragging 
		TimeLine t = pres.ctl;
		boolean found = false;
		for(int i=t.numElements()-1; i>=0; i--) {
			Element el = t.getElement(i);
			boolean maybe_resizing = false;

			// check for click
			int s = el.getStyle();
			boolean gotclick = false;
			if (s == 0) {
				Rectangle b = el.getBounds();
				if (b.inside(x,y)) {
					gotclick = true;
					if (x > b.x+b.width-5 &&
					         y > b.y+b.height-5) {
						// resizing width and height
						maybe_resizing = true;
						resizemode = 0;
						resizex = b.x+b.width - gx;
						resizey = b.y+b.height - gy;
						}
					else if (x > b.x+b.width-5) {
						// resizing width
						maybe_resizing = true;
						resizemode = 1;
						resizex = b.x+b.width - gx;
						}
					else if (y > b.y+b.height-5) {
						// resizing height
						maybe_resizing = true;
						resizemode = 2;
						resizey = b.y+b.height - gy;
						}
					}
				}
			else if (s == 1) {
				Point ep[] = el.getEndPoints();
				int d0 = Math.abs(x-ep[0].x) +
					 Math.abs(y-ep[0].y);
				int d1 = Math.abs(x-ep[1].x) +
					 Math.abs(y-ep[1].y);
				if (d0 < 7) {
					// click on 1st point to move
					gotclick = true;
					}
				else if (d1 < 7) {
					// click on 2nd point to adjust
					gotclick = maybe_resizing = true;
					resizemode = 1;
					resizex = ep[1].x-gx;
					resizey = ep[1].y-gy;
					}
				}

			if (gotclick) {
				// found clicked element..
				found = true;
				if (el.locked != null) continue;
				boolean shift_click =
					((e.modifiers & Event.SHIFT_MASK) != 0);
				if (selelm.indexOf(el) != -1) {
					// drag, edit or resize?
					long now = System.currentTimeMillis();
					if (shift_click) {
						// just de-select
						selelm.removeElement(el);
						}
					else if (now - lastclick < 500) {
						// pop up editor for
						// selected element
						selelm.removeAllElements();
						openEditWindow(el);
						}
					else if (maybe_resizing) {
						// start resizing
						resizing = true;
						}
					else {
						// start moving this element
						moving = true;
						moveelm = el;
						movex = gx-el.x;
						movey = gy-el.y;
						movestartx = gx;
						movestarty = gy;
						}
					lastclick = now;

					if (resizing) {
						selelm.removeAllElements();
						selelm.addElement(el);
						}
					}
				else {
					if (!shift_click) {
						// select this element only
						selelm.removeAllElements();
						selelm.addElement(el);
						}
					else {
						// add this element to
						// the selection
						selelm.addElement(el);
						}
					}
				repaint();
				break;
				}
			}
		if (!found) selelm.removeAllElements();
		repaint();
		}
	else if (mode == CREATE_RECT || mode == CREATE_LINE || 
	         mode == SELECT) {
		// mark start of drag
		selelm.removeAllElements();
		parent.noneSelected();
		dragging = true;
		dragx1 = gx; dragy1 = gy;
		}
	else if (mode == CREATE_FREE) {
		// record first freehand point
		freehand = true;
		freepts = new Vector();
		freepts.addElement(new Point(gx, gy));
		dragx1 = gx; dragy1 = gy;
		}
	else if (mode == CREATE_POLY) {
		// record polygon point
		if (polygon) {
			Point lp = (Point)polypts.lastElement();
			if (Math.abs(lp.x - gx) + Math.abs(lp.y - gy) < 5) {
				// done polygon
				if (defining) {
					defining = false;
					definePath(polypts);
					}
				else if (polypts.size() > 2) {
					Element ne = (Element)
						Util.createObject(create_class);
					for(int i=0; i<polypts.size(); i++) {
						Point p = (Point)
							polypts.elementAt(i);
						p.x -= dragx1; p.y -= dragy1;
						}
					ne.create(polypts, parent.ch_col.fore,
					                   parent.ch_col.back);
					newElement(ne, dragx1, dragy1);
					}
				polygon = false;
				}
			else {
				if (x < dragx1) dragx1 = gx;
				if (y < dragy1) dragy1 = gy;
				polypts.addElement(new Point(gx, gy));
				}
			}
		else {
			polygon = true;
			polypts = new Vector();
			polypts.addElement(new Point(gx, gy));
			dragx1 = gx; dragy1 = gy;
			dragx2 = gx; dragy2 = gy;
			}
		repaint();
		}
	updateMenus();
	return true;
	}

	public boolean mouseDrag(Event e, int x, int y)
	{
	x += tx; y += ty;
	if (gridmode) {
		x = (x/gridw)*gridw;
		y = (y/gridh)*gridh;
		}
	if (dragging) {
		// continue to mark out drag area
		if (mode == CREATE_LINE) {
			dragx2 = x; dragy2 = y;
			}
		else {
			dragx2 = x<dragx1 ? dragx1 : x;
			dragy2 = y<dragy1 ? dragy1 : y;
			}
		repaint();
		return true;
		}
	else if (freehand) {
		// add another free point
		Point lp = (Point)freepts.lastElement();
		if (lp.x != x || lp.y != y) {
			freepts.addElement(new Point(x, y));
			if (x < dragx1) dragx1 = x;
			if (y < dragy1) dragy1 = y;
			repaint();
			}
		return true;
		}
	else if (moving) {
		// drag all the selected elements.. actual addition of a new
		// position is only done on mouseUp()
		int mx = x-movex-moveelm.x, my = y-movey-moveelm.y;
		for(int i=0; i<selelm.size(); i++) {
			Element el = (Element)selelm.elementAt(i);
			el.x += mx; el.y += my;
			}
		repaint();
		return true;
		}
	else if (resizing) {
		// resizing one element
		Element re = (Element)selelm.elementAt(0);
		int s = re.getStyle();
		if (s == 0) {
			Rectangle r = re.getBounds();
			x = x<r.x ? r.x : x;
			y = y<r.y ? r.y : y;
			if (resizemode == 0)
				re.setSize(x-r.x+resizex, y-r.y+resizey);
			else if (resizemode == 1)
				re.setSize(x-r.x+resizex, r.height);
			else
				re.setSize(r.width, y-r.y+resizey);
			}
		else if (s == 1) {
			Point ep[] = re.getEndPoints();
			re.setSize(x-ep[0].x-resizex, y-ep[0].y-resizey);
			}
		repaint();
		return true;
		}
	return false;
	}

	public boolean mouseMove(Event e, int x, int y)
	{
	x += tx; y += ty;
	if (gridmode) {
		x = (x/gridw)*gridw;
		y = (y/gridh)*gridh;
		}
	if (polygon) {
		dragx2 = x; dragy2 = y;
		repaint();
		return true;
		}
	return false;
	}

	public boolean mouseUp(Event e, int x, int y)
	{
	x += tx; y += ty;
	if (gridmode) {
		x = (x/gridw)*gridw;
		y = (y/gridh)*gridh;
		}
	if (dragging) {
		// finished dragging.. create or select
		if (mode == CREATE_RECT || mode == CREATE_LINE) {
			// create a new element
			Element ne = (Element)Util.createObject(create_class);
			Dimension d = new Dimension(dragx2-dragx1,
			                            dragy2-dragy1);
			ne.create(d, parent.ch_col.fore, parent.ch_col.back);
			newElement(ne, dragx1, dragy1);
			}
		else if (mode == SELECT) {
			// select some elements
			TimeLine t = pres.ctl;
			Rectangle selr = new Rectangle(
				dragx1, dragy1, dragx2-dragx1, dragy2-dragy1);
			for(int i=0; i<t.numElements(); i++) {
				Element el = t.getElement(i);
				int s = el.getStyle();
				if (s == 0 && el.getBounds().intersects(selr))
						selelm.addElement(el);
				else if (s == 1) {
					Point ep[] = el.getEndPoints();
					if (selr.inside(ep[0].x, ep[0].y) ||
					    selr.inside(ep[1].x, ep[1].y))
						selelm.addElement(el);
					 }
				}
			if (selelm.size() != 0) {
				parent.controls.arrow.select();
				pointMode();
				parent.someSelected();
				}
			}
		dragging = false;
		repaint();
		return true;
		}
	else if (freehand && defining) {
		// Finished defining a path
		freehand = defining = false;
		definePath(freepts);
		repaint();
		return true;
		}
	else if (freehand) {
		// Finished freehand drawing..
		Element ne = (Element)Util.createObject(create_class);
		for(int i=0; i<freepts.size(); i++) {
			Point p = (Point)freepts.elementAt(i);
			p.x -= dragx1; p.y -= dragy1;
			}
		ne.create(freepts, parent.ch_col.fore, parent.ch_col.back);
		newElement(ne, dragx1, dragy1);
		freehand = false;
		repaint();
		return true;
		}
	else if (moving) {
		// Finished moving elements.. If a moved element is at one
		// of it's tweening points, change that point. However, if it
		// is between points, add a new one.
		int now = pres.ctl.time;
		moving = false;
		if (Math.abs(x - movestartx) + Math.abs(y - movestarty) < 5) {
			// ignore this small move
			pres.ctl.setTime(now);
			repaint();
			return false;
			}
		for(int i=0; i<selelm.size(); i++) {
			Element me = (Element)selelm.elementAt(i);
			ElementPos ep = me.tweenPos(now)[0];
			if (ep.time == now) {
				// Change this tween point..
				ep.x = me.x; ep.y = me.y;
				}
			else {
				// Create a new tweening point..
				me.addPosition(
				  new ElementPos(now, me.x, me.y, true));
				}
			}
		}
	else if (resizing) {
		// done resizing some element
		resizing = false;
		}
	return false;
	}

	public void reshape(int nx, int ny, int nw, int nh)
	{
	super.reshape(nx, ny, nw, nh);
	if (hsl != null && vsl != null && pres != null) {
		vsl.setValues(0, nh-hsl.size().height, pres.h);
		hsl.setValues(0, nw-vsl.size().width, pres.w);
		tx = vsl.getValue();
		ty = hsl.getValue();
		}
	repaint();
	}

	public Dimension preferredSize()
	{
	return new Dimension(515, 415);
	}

	public Dimension minimumSize()
	{
	return preferredSize();
	}

	public void moved(CbScrollbar sb, int v)
	{
	if (sb == hsl) tx = v;
	else if (sb == vsl) ty = v;
	repaint();
	}

	public void moving(CbScrollbar sb, int v)
	{
	moved(sb, v);
	}

	private void newElement(Element e, int x, int y)
	{
	e.addPosition(new ElementPos(pres.ctl.time, x, y, true));
	pres.nameElement(e);
	pres.ctl.addElement(e);
	e.setTime(pres.ctl.time);
	}

	/**Sets the positions of all selected elements to follow the
	 * given path
	 */
	private void definePath(Vector p)
	{
	Point p0 = (Point)p.elementAt(0);
	for(int j=0; j<selelm.size(); j++) {
		Element e = (Element)selelm.elementAt(j);
		e.pos = new Vector();
		int d = define_end - define_start;
		int sz = p.size(), step = 1;
		if (sz > 50) step = p.size()/50;
		for(int i=0; i<sz; i+=step) {
			Point pt = (Point)p.elementAt(i);
			e.addPosition(new ElementPos(define_start+(i*d/sz),
			               pt.x-(p0.x-e.x), pt.y-(p0.y-e.y), true));
			}
		}
	pres.ctl.setTime(pres.ctl.time);
	repaint();
	pointMode();
	}

	private void openEditWindow(Element el)
	{
	String ec = el.getClass().getName()+"Editor";
	ElementEditor ed = (ElementEditor)Util.createObject(ec);
	ed.edit(el, this);
	}

	public void actionsWindowClosed(Object o) { actionswin = null; }
	public Vector listActions(Object o) { return pres.ctl.acts; }
	public void setActions(Object o, Vector a) { pres.ctl.acts = a; }
	public int[] getEvents(Object o)
	{
	int elist[] = { 0, 1 };
	return elist;
	}
}

/**A window for choosing which elements to select
 */
class ElementChooser extends FixedFrame implements CbButtonCallback,Runnable
{
	EditArea parent;
	CbButton close;
	List list;
	Thread th;
	Vector v = new Vector(), sv = new Vector();

	ElementChooser(EditArea p)
	{
	super(300, 300);
	parent = p;
	parent.chooser = this;
	setLayout(new BorderLayout());
	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(close = new CbButton("Close", this));
	add("South", bot);
	
	Panel mid = new BorderPanel(1);
	mid.setLayout(new GridLayout(1,1));
	mid.add(list = new List());
	list.setMultipleSelections(true);
	add("Center", mid);

	add("North", new Label("Select Elements"));
	pack();
	show();
	setTitle("Select Elements");
	(th = new Thread(this)).start();
	}

	/**Synchronize the list with the currently selected elements
	 */
	public void run()
	{
	while(true) {
		// Check for changes in the list of elements
		TimeLine t = parent.pres.ctl;
		Vector nv = new Vector();
		for(int i=0; i<t.elmc; i++)
			if (parent.lockelm.indexOf(t.elm[i]) < 0)
				nv.addElement(t.elm[i]);
		boolean changed = (nv.size() != v.size());
		if (!changed) {
			for(int i=0; i<nv.size(); i++)
				if (nv.elementAt(i) != v.elementAt(i))
					changed = true;
			}
		v = nv;
		if (changed) {
			// Need to re-fill the list
			list.clear();
			for(int i=0; i<v.size(); i++)
				list.addItem(((Element)v.elementAt(i)).name);
			}

		// Check if the selection list has changed
		Vector nsv = parent.selelm;
		boolean schanged = (nsv.size() != sv.size());
		if (!schanged) {
			for(int i=0; i<nsv.size(); i++)
				if (nsv.elementAt(i) != sv.elementAt(i))
					schanged = true;
			}
		sv.removeAllElements();
		for(int i=0; i<nsv.size(); i++)
			sv.addElement(nsv.elementAt(i));

		// Select elements that are selected
		if (changed || schanged) {
			for(int i=0; i<v.size(); i++) {
				Element e = (Element)v.elementAt(i);
				boolean s = list.isSelected(i),
					ss = (parent.selelm.indexOf(e) >= 0);
				if (ss && !s) list.select(i);
				else if (!ss && s) list.deselect(i);
				}
			}

		try { Thread.sleep(500); } catch(Exception e) { }
		}
	}

	public boolean handleEvent(Event evt)
	{
	if (evt.target == list) {
		if (evt.id == Event.LIST_SELECT) {
			int i = ((Integer)evt.arg).intValue();
			parent.selelm.addElement(v.elementAt(i));
			}
		else if (evt.id == Event.LIST_DESELECT) {
			int i = ((Integer)evt.arg).intValue();
			parent.selelm.removeElement(v.elementAt(i));
			}
		if (parent.selelm.size() == 0)
			parent.parent.noneSelected();
		else
			parent.parent.someSelected();
		parent.repaint();
		return true;
		}
	return super.handleEvent(evt);
	}

	public void click(CbButton b)
	{
	if (b == close) {
		parent.chooser = null;
		dispose();
		}
	}

	public void dispose()
	{
	super.dispose();
	th.stop();
	}
}
