import java.awt.*;

class MoveAction extends Action
{
	String elm;
	int xmode = 0, ymode = 0, vmode = 0;
	int pos, x, y;
	boolean visible;

	void activate(Presentation p)
	{
	Element e = p.findElement(elm);
	if (e == null) {
		System.out.println("Failed to find element "+elm);
		return;
		}
	if (pos < 0 || pos >= e.pos.size()) {
		System.out.println("Failed to find position "+pos);
		return;
		}
	ElementPos ep = (ElementPos)e.pos.elementAt(pos);
	if (xmode == 1) ep.x = x;
	else if (xmode == 2) ep.x += x;
	if (ymode == 1) ep.y = y;
	else if (ymode == 2) ep.y += y;
	if (vmode == 1) ep.visible = visible;
	p.changed();
	}

	HData save()
	{
	HData hd = super.save();
	hd.addString("elm", elm);
	hd.addInteger("pos", pos);
	hd.addInteger("x", x); hd.addInteger("y", y);
	hd.addBoolean("visible", visible);
	hd.addInteger("xmode", xmode);
	hd.addInteger("ymode", ymode);
	hd.addInteger("vmode", vmode);
	return hd;
	}

	void load(HData hd)
	{
	super.load(hd);
	elm = hd.getString("elm");
	pos = hd.getInteger("pos");
	x = hd.getInteger("x"); y = hd.getInteger("y");
	visible = hd.getBoolean("visible");
	xmode = hd.getInteger("xmode");
	ymode = hd.getInteger("ymode");
	vmode = hd.getInteger("vmode");
	}
}


class MoveActionEditor extends ActionEditor
{
	MoveAction s;
	Choice elm, pos;
	TextField xval, yval;
	Choice xmode, ymode, visible;

	protected void makeUI()
	{
	super.makeUI();
	s = (MoveAction)act;
	left.add(new Label("Element:"));
	right.add(elm = new Choice());
	Element el[] = parent.pres.listElements();
	for(int i=0; i<el.length; i++)
		elm.addItem(el[i].name);
	if (el.length == 0) elm.addItem("<None>");
	if (s.elm != null) elm.select(s.elm);
	left.add(new Label("Time:"));
	right.add(pos = new Choice());
	fillPositions();
	pos.select(s.pos);
	left.add(new Label("X value:"));
	right.add(xval = new TextField(String.valueOf(s.x)));
	left.add(new Label("Change X:"));
	right.add(xmode = new Choice());
	xmode.addItem("No"); xmode.addItem("Absolute");
	xmode.addItem("Relative"); xmode.select(s.xmode);
	left.add(new Label("Y value:"));
	right.add(yval = new TextField(String.valueOf(s.y)));
	left.add(new Label("Change Y:"));
	right.add(ymode = new Choice());
	ymode.addItem("No"); ymode.addItem("Absolute");
	ymode.addItem("Relative"); ymode.select(s.ymode);
	left.add(new Label("Visible:"));
	right.add(visible = new Choice());
	visible.addItem("Don't change");
	visible.addItem("Yes");
	visible.addItem("No");
	visible.select(s.vmode==0 ? 0 : s.visible ? 1 : 2);
	}

	public boolean action(Event evt, Object obj)
	{
	if (evt.target == elm) {
		fillPositions();
		return true;
		}
	return false;
	}

	private void fillPositions()
	{
	Font posf = pos.getFont();
	right.remove(pos);
	right.add(pos = new Choice(), 1); pos.setFont(posf);
	right.layout();
	Element e = parent.pres.findElement(elm.getSelectedItem());
	if (e != null) {
		for(int i=0; i<e.pos.size(); i++) {
			ElementPos ep = (ElementPos)e.pos.elementAt(i);
			pos.addItem(String.valueOf(ep.time));
			}
		if (e.pos.size() == 0)
			pos.addItem("<None>");
		}
	}

	protected boolean apply()
	{
	String e;
	int p, x, y, xm, ym, vm;
	boolean v;
	Element ep;
	if ((ep = parent.pres.findElement(e = elm.getSelectedItem())) == null) {
		new ErrorWindow("Element "+e+" not found");
		return false;
		}
	if ((p = pos.getSelectedIndex()) >= ep.pos.size()) {
		new ErrorWindow("Element position "+p+" not found");
		return false;
		}
	xm = xmode.getSelectedIndex();
	if ((x = getNum(xval)) == -Integer.MAX_VALUE) {
		new ErrorWindow("X value must be a number");
		}
	ym = ymode.getSelectedIndex();
	if ((y = getNum(yval)) == -Integer.MAX_VALUE) {
		new ErrorWindow("Y value must be a number");
		}
	vm = visible.getSelectedIndex();
	if (vm == 0) { v = false; vm = 0; }
	else { v = (vm==1); vm = 1; }
	if (!super.apply()) return false;
	s.elm = e; s.pos = p;
	s.xmode = xm; s.ymode = ym; s.vmode = vm;
	s.x = x; s.y = y; s.visible = v;
	return true;
	}
}
