import java.awt.*;
import java.util.*;

/**A block of text, wrapped to fit into some rectangle. The text can
 * contain return characters and repeated spaces, which will be preserved
 * when rendered (if possible)
 */
class TextElement extends Element
{
	int w,h;
	boolean trans = true;
	String txt = "Type some text here";
	String txtw[];
	Font font = new Font("TimesRoman", Font.PLAIN, 24);
	int align;

	TextElement()
	{
	splitWords();
	}

	/**Split the text string into words, spaces and return characters
	 */
	void splitWords()
	{
	int l = txt.length();
	int mode = 0;
	StringBuffer str = null;
	Vector txtv = new Vector();
	for(int i=0; i<l; i++) {
		char c = txt.charAt(i);
		if (mode == 0) {
			// anything can happen
			if (c == ' ') {
				mode = 1;
				str = new StringBuffer(String.valueOf(c));
				}
			else if (c == '\n')
				txtv.addElement(String.valueOf(c));
			else {
				mode = 2;
				str = new StringBuffer(String.valueOf(c));
				}
			}
		else if (mode == 1) {
			if (c == ' ') str.append(c);
			else {
				txtv.addElement(str);
				str = null;
				mode = 0;
				i--;
				}
			}
		else if (mode == 2) {
			if (c == ' ' || c == '\n') {
				txtv.addElement(str);
				str = null;
				mode = 0;
				i--;
				}
			else str.append(c);
			}
		}
	if (str != null) txtv.addElement(str);
	txtw = new String[txtv.size()];
	for(int i=0; i<txtw.length; i++)
		txtw[i] = txtv.elementAt(i).toString();
	}

	void paint(Graphics gg, boolean e)
	{
	Graphics g = gg.create(x, y, w, h);
	if (!trans) {
		g.setColor(bcol);
		g.fillRect(0, 0, w, h);
		}
	g.setFont(font); g.setColor(fcol);
	FontMetrics fnm = g.getFontMetrics();
	int tx = 0, fh = fnm.getHeight(), ty = fh-fnm.getDescent();
	String line = "";
	for(int i=0; i<txtw.length; i++) {
		char c = txtw[i].charAt(0);
		int tw = fnm.stringWidth(txtw[i]);
		if (c == '\n') {
			// force wrap to next line
			drawLine(g, line, ty, tx);
			tx = 0; ty += fh; line = "";
			}
		else {
			if (c != ' ' && tx+tw > w && tx != 0) {
				// wrap to next line
				drawLine(g, line, ty, tx);
				tx = 0; ty += fh; line = "";
				}
			line += txtw[i];
			tx += tw;
			}
		}
	drawLine(g, line, ty, tx);
	}

	private void drawLine(Graphics g, String l, int y, int tw)
	{
	if (align == 0) g.drawString(l, 0, y);
	else if (align == 1) g.drawString(l, (w-tw)/2, y);
	else g.drawString(l, w-tw, y);
	}

	void create(Dimension d, Color f, Color b)
	{
	setSize(d.width, d.height);
	fcol = f; bcol = b;
	}

	Rectangle getBounds()
	{
	return new Rectangle(x, y, w, h);
	}

	void setSize(int nw, int nh)
	{
	w = nw; h = nh;
	}

	void copy(Element t)
	{
	TextElement tt = (TextElement)t;
	tt.w = w; tt.h = h;
	tt.trans = trans;
	tt.txt = txt; tt.splitWords();
	tt.font = font;
	tt.align = align;
	super.copy(t);
	}

	HData save()
	{
	HData hd = super.save();
	hd.addInteger("w", w); hd.addInteger("h", h);
	hd.addBoolean("trans", trans);
	hd.addFont("font", font);
	hd.addString("txt", txt);
	hd.addInteger("align", align);
	return hd;
	}

	void load(HData hd)
	{
	super.load(hd);
	w = hd.getInteger("w"); h = hd.getInteger("h");
	trans = hd.getBoolean("trans");
	font = hd.getFont("font");
	txt = hd.getString("txt");
	align = hd.getInteger("align");
	splitWords();
	}
}

