import java.awt.*;
import java.util.*;

class TextElementEditor extends ElementEditor
{
	TextField width, height, size;
	CbButton textb;
	TextElementWindow textwin;
	String text;
	Checkbox trans, ntrans;
	Checkbox bold, italic;
	Choice align;
	Choice font;
	CbColorButton fcol, bcol;
	TextElement t;

	protected void makeUI()
	{
	super.makeUI();
	t = (TextElement)elm;
	pan.setLayout(new GridLayout(0, 2));
	pan.add(new Label("Width:"));
	pan.add(width = new TextField(String.valueOf(t.w)));
	pan.add(new Label("Height:"));
	pan.add(height = new TextField(String.valueOf(t.h)));
	pan.add(new Label("Text:"));
	pan.add(textb = new CbButton("Edit...", this));
	text = t.txt;

	pan.add(new Label("Font:"));
	pan.add(font = new Choice());
	String flist[] = Util.tk.getFontList();
	for(int i=0; i<flist.length; i++) font.addItem(flist[i]);
	font.select(t.font.getName());
	pan.add(new Label("Font size:"));
	pan.add(size = new TextField(String.valueOf(t.font.getSize())));

	pan.add(new Label("Font style:"));
	Panel sp = new GrayPanel();
	sp.setLayout(new GridLayout(1, 2));
	sp.add(bold = new Checkbox("Bold"));
	bold.setState(t.font.isBold());
	sp.add(italic = new Checkbox("Italic"));
	italic.setState(t.font.isItalic());
	pan.add(sp);

	pan.add(new Label("Alignment:"));
	pan.add(align = new Choice());
	align.addItem("Left");
	align.addItem("Center");
	align.addItem("Right");
	align.select(t.align);

	pan.add(new Label("Foreground:"));
	pan.add(fcol = new CbColorButton(t.fcol));
	pan.add(new Label("Background:"));
	pan.add(bcol = new CbColorButton(t.bcol));
	pan.add(new Label("Transparent:"));
	Panel tp = new GrayPanel();
	tp.setLayout(new GridLayout(1, 2));
	CheckboxGroup tg = new CheckboxGroup();
	tp.add(trans = new Checkbox("Yes", tg, t.trans));
	tp.add(ntrans = new Checkbox("No", tg, !t.trans));
	pan.add(tp);
	}

	protected boolean apply()
	{
	int w, h;
	if ((w = getNum(width)) <= 0) {
		new ErrorWindow("Width must be a positive number");
		return false;
		}
	if ((h = getNum(height)) <= 0) {
		new ErrorWindow("Height must be a positive number");
		return false;
		} 
	String n = font.getSelectedItem();
	int s;
	if ((s = getNum(size)) <= 0 || s > 72) {
		new ErrorWindow("Font size must be a positive number "+
		                "less than 72");
		return false;
		}
	int i = (bold.getState() ? Font.BOLD : 0) |
	        (italic.getState() ? Font.ITALIC : 0);
	int a = align.getSelectedIndex();
	boolean r = trans.getState();
	Color f = fcol.col, b = bcol.col;
	if (!super.apply()) return false;
	t.w = w; t.h = h;
	t.fcol = f; t.bcol = b;
	t.trans = r; t.font = new Font(n, i, s);
	t.txt = text; t.splitWords();
	t.align = a;
	return true;
	}

	/**Change font in text window if the font, size or style are changed
	 */
	public boolean action(Event evt, Object obj)
	{
	Object tar = evt.target;
	if ((tar == font || tar == size || tar == bold || tar == italic) &&
	    textwin != null)
		textwin.tarea.setFont(new Font(
			font.getSelectedItem(),
			(bold.getState() ? Font.BOLD : 0) |
			(italic.getState() ? Font.ITALIC : 0),
			Integer.parseInt(size.getText())));
	return true;
	}

	public void click(CbButton b)
	{
	super.click(b);
	if (b == textb && textwin == null)
		new TextElementWindow(this);
	}
}


class TextElementWindow extends Frame implements CbButtonCallback
{
	CbButton ok, cancel;
	TextArea tarea;
	TextElementEditor parent;

	TextElementWindow(TextElementEditor p)
	{
	parent = p;
	parent.textwin = this;
	setLayout(new BorderLayout());
	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(ok = new CbButton("Ok", this));
	bot.add(cancel = new CbButton("Cancel", this));
	add("South", bot);
	StringTokenizer tok = new StringTokenizer(parent.text, "\n");
	int r, c = 0;
	for(r=0; tok.hasMoreTokens(); r++) {
		int l = tok.nextToken().length();
		if (l > c) c = l;
		}
	r = r < 3 ? 3 : r; c = c < 16 ? 16 : c;

	add("Center", tarea = new TextArea(parent.text, r, c));
	tarea.setFont(new Font(
		parent.font.getSelectedItem(),
		(parent.bold.getState() ? Font.BOLD : 0) |
		(parent.italic.getState() ? Font.ITALIC : 0),
		Integer.parseInt(parent.size.getText())));

	pack();
	show();
	setTitle("Edit Text");
	}

	public void click(CbButton b)
	{
	if (b == ok)
		parent.text = tarea.getText();
	if (b == ok || b == cancel) {
		dispose();
		parent.textwin = null;
		}
	}
}

