import java.awt.*;
import java.util.*;

class TimeLineEditor extends FixedFrame implements CbButtonCallback
{
	jshock parent;
	TimeLine tl;
	boolean new_timeline = false;
	CbButton ok, cancel, delete;
	TextField name, length, position, speed;
	Choice fill;
	CbColorButton fillcol;
	CbImageChooser fillimg;
	Checkbox storeu, stored;

	TimeLineEditor(jshock p)
	{
	this(p, new TimeLine(p.edit_area.pres, 5000, "New"));
	new_timeline = true;
	}

	TimeLineEditor(jshock p, TimeLine t)
	{
	parent = p;
	tl = t;
	parent.tleditor = this;
	setLayout(new BorderLayout());

	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(ok = new CbButton("Ok", this));
	if (!new_timeline)
		bot.add(delete = new CbButton("Delete", this));
	bot.add(cancel = new CbButton("Cancel", this));
	add("South", bot);

	Panel mid = new BorderPanel(1);
	mid.setLayout(new GridLayout(0, 2));
	mid.add(new Label("Name:"));
	mid.add(name = new TextField(tl.name, 15));
	mid.add(new Label("Length:"));
	Panel lp = new GrayPanel();
	lp.setLayout(new BorderLayout());
	lp.add("Center", length = new TextField(String.valueOf(tl.len), 6));
	lp.add("East", new Label("ms"));
	mid.add(lp);
	mid.add(new Label("Position:"));
	Panel pp = new GrayPanel();
	pp.setLayout(new BorderLayout());
	pp.add("Center", position = new TextField(String.valueOf(tl.time), 6));
	pp.add("East", new Label("ms"));
	mid.add(pp);
	mid.add(new Label("Speedup:"));
	mid.add(speed = new TextField(String.valueOf(tl.speed)));

	mid.add(new Label("Background:"));
	mid.add(fill = new Choice());
	fill.addItem("Solid Color");
	fill.addItem("Tiled Image");
	fill.addItem("Scaled Image");
	fill.select(tl.fillimg==null ? 0 : tl.fillimg_tile ? 1 : 2);
	mid.add(new Label("Color:"));
	mid.add(fillcol = new CbColorButton(tl.fillcol));
	mid.add(new Label("Image:"));
	mid.add(fillimg = new CbImageChooser(tl.fillimg));
	mid.add(new Label("Store Image:"));
	Panel up = new GrayPanel();
	up.setLayout(new GridLayout(1, 2));
	CheckboxGroup ug = new CheckboxGroup();
	up.add(storeu = new Checkbox("URL", ug,
                        tl.fillimg==null || tl.fillimgsrc!=null));
	up.add(stored = new Checkbox("Data", ug, !storeu.getState()));
	mid.add(up);
	add("Center", mid);

	pack();
	show();
	setTitle("Edit Timeline");
	Util.recursiveBackground(this, Color.lightGray);
	}

	public void click(CbButton b)
	{
	if (b == ok) {
		String n;
		int l, p, m;
		float s;
		Color c = null;
		Image i = null;
		if ((n = name.getText().trim()).equals("")) {
			new ErrorWindow("Timeline name must not be empty");
			return;
			}
		Vector tlist = parent.edit_area.pres.tl;
		for(int j=0; j<tlist.size(); j++) {
			TimeLine ot = (TimeLine)tlist.elementAt(j);
			if (ot != tl && ot.name.equals(n)) {
				new ErrorWindow("A timeline called "+n+
						"already exists");
				return;
				}
			}
		if ((l = getNum(length)) <= 0) {
			new ErrorWindow("Length must be a positive number");
			return;
			}
		if ((p = getNum(position)) < 0) {
			new ErrorWindow("Position must be a positive number");
			return;
			}
		try { s = Float.valueOf(speed.getText()).floatValue(); }
		catch(NumberFormatException e) { s = -1; }
		if (s <= 0) {
			new ErrorWindow("Speed must be a positive number");
			return;
			}
		String r = i==null || stored.getState() ? null : fillimg.imgsrc;
		if (storeu.getState() && i != null) {
			if (r == null) {
				new ErrorWindow(
					"Only images for which the source "+
					"is known can be stored as URLs");
				return;
				}
			else if (!r.startsWith("http:") &&
			         !r.startsWith("https:")) {
				new ErrorWindow("Only images loaded from a "+
				                "URL can be stored as URLs");
				return;
				}
			}

		// apply inputs
		tl.name = n;
		tl.len = l; tl.setTime(p);
		tl.speed = s;
		m = fill.getSelectedIndex();
		tl.fillcol = fillcol.col;
		tl.fillimgsrc = r;
		if (m == 0)
			tl.fillimg = null;
		else {
			tl.fillimg = fillimg.img;
			if (tl.fillimg != null) {
				tl.fillimg_w = Util.getWidth(tl.fillimg);
				tl.fillimg_h = Util.getHeight(tl.fillimg);
				tl.fillimg_tile = (m == 1);
				}
			}
		if (parent.edit_area.pres.ctl == tl) {
			// this timeline is the active one
			parent.time_line.setMaximum(l);
			parent.gotoTime(p);
			}
		else if (new_timeline) {
			// creating a new timeline
			parent.edit_area.pres.tl.addElement(tl);
			}
		parent.showTimeLines();
		}
	else if (b == delete) {
		// delete this timeline
		Vector tlist = parent.edit_area.pres.tl;
		if (tlist.size() == 1)
			new ErrorWindow("This is the only timeline");
		else {
			tlist.removeElement(tl);
			parent.showTimeLines();
			if (parent.edit_area.pres.ctl == tl)
				parent.edit_area.gotoTimeLine(0);
			}
		}
	if (b == ok || b == cancel || b == delete)
		dispose();
	}

	public void dispose()
	{
	parent.tleditor = null;
	super.dispose();
	}

	protected int getNum(TextField t)
	{
	try {
		return Integer.parseInt(t.getText());
		}
	catch(Exception e) {
		return -1;
		}
	}
}
