import java.awt.*;
import java.util.*;
import java.io.*;

/**Each jshock object is a top-level window that is either empty, or is
 * editing some jshock presentation.
 */
public class jshock extends FixedFrame implements CbSliderCallback
{
	static int wincount = 0;
	MenuBar menu_bar;
	Menu file_menu, edit_menu, timeline_menu, tools_menu;
	MenuItem new_mitem, open_mitem, save_mitem, saveas_mitem,
		 close_mitem, exit_mitem;
	MenuItem cut_mitem, copy_mitem, paste_mitem, del_mitem,
		 selall_mitem, raise_mitem, lower_mitem, edit_mitem, list_mitem;
	MenuItem tledit_mitem, tlactions_mitem, tlnew_mitem;
	CheckboxMenuItem tlgoto_mitem[];
	MenuItem resize_mitem, run_mitem, path_mitem, grid_mitem;
	CheckboxMenuItem gridon_mitem;
	EditControls controls;
	EditArea edit_area;
	CbSlider time_line;
	TextField time_pos;
	ColorChooser ch_col;
	static Element cutbuf[] = new Element[0];
	TimeLineEditor tleditor;
	String file_name;
	int gridw = 10, gridh = 10;
	boolean gridon;
	GridWindow gridwin;
	PathWindow pathwin;

	/**Parse command line parameters, and create an empty window or
	 * one containing a presentation.
	 */
	public static void main(String argv[])
	{
	String file = null;

	for(int i=0; i<argv.length; i++) {
		file = argv[i];
		}
	if (file == null)
		new jshock(500, 400);
	else
		new jshock(file);
	}


	/**Create a new presentation window, and load the given file into it
	 */
	jshock(String file)
	{
	wincount++;
	createUI();
	if (loadFile(file))
		save_mitem.enable();
	else {
		System.err.println("Failed to load "+file);
		dispose();
		}
	}


	/**Create a new presentation window editing an empty presentation
	 */
	jshock(int w, int h)
	{
	wincount++;
	createUI();
	edit_area.setPresentation(new Presentation(5000, w, h));
	setTitle("Jshock ("+w+" x "+h+")");
	}


	/**Create the user interface
	 */
	void createUI()
	{
	// Create menu bar
	file_menu = new Menu("File");
	edit_menu = new Menu("Edit");
	timeline_menu = new Menu("Timeline");
	tools_menu = new Menu("Tools");
	menu_bar = new MenuBar();
	menu_bar.add(file_menu);
	menu_bar.add(edit_menu);
	menu_bar.add(timeline_menu);
	menu_bar.add(tools_menu);
	setMenuBar(menu_bar);

	// Create file menu items
	file_menu.add(new_mitem = new MenuItem("New.."));
	file_menu.add(open_mitem = new MenuItem("Open.."));
	file_menu.add(save_mitem = new MenuItem("Save"));
	save_mitem.disable();
	file_menu.add(saveas_mitem = new MenuItem("Save As.."));
	file_menu.add(close_mitem = new MenuItem("Close"));
	file_menu.addSeparator();
	file_menu.add(exit_mitem = new MenuItem("Exit"));

	//  Create edit menu items
	edit_menu.add(cut_mitem = new MenuItem("Cut"));
	edit_menu.add(copy_mitem = new MenuItem("Copy"));
	edit_menu.add(paste_mitem = new MenuItem("Paste"));
	edit_menu.add(del_mitem = new MenuItem("Delete"));
	edit_menu.add(selall_mitem = new MenuItem("Select All"));
	edit_menu.addSeparator();
	edit_menu.add(raise_mitem = new MenuItem("Raise"));
	edit_menu.add(lower_mitem = new MenuItem("Lower"));
	edit_menu.add(edit_mitem = new MenuItem("Edit.."));
	edit_menu.add(list_mitem = new MenuItem("List.."));

	// Create timeline menu
	timeline_menu.add(tledit_mitem = new MenuItem("Edit.."));
	timeline_menu.add(tlactions_mitem = new MenuItem("Actions.."));
	timeline_menu.add(tlnew_mitem = new MenuItem("New.."));
	timeline_menu.addSeparator();

	// Create tools menu items
	tools_menu.add(path_mitem = new MenuItem("Define Path.."));
	tools_menu.add(run_mitem = new MenuItem("Run Presentation.."));
	tools_menu.add(resize_mitem = new MenuItem("Change Size.."));
	tools_menu.add(grid_mitem = new MenuItem("Grid Size.."));
	tools_menu.add(gridon_mitem = new CheckboxMenuItem("Grid"));

	// Create edit controls and editing area
	setLayout(new BorderLayout());
	Panel cpan = new GrayPanel();
	cpan.setLayout(new BorderLayout());
	cpan.add("North", controls = new EditControls(this));
	// cpan.add("Center", ch_thick = new ThicknessChooser(this));
	cpan.add("South", ch_col = new ColorChooser(this));
	add("West", cpan);
	add("Center", edit_area = new EditArea(this));
	Panel top = new GrayPanel();
	top.setLayout(new BorderLayout());
	top.add("Center", time_line = new CbSlider(0, 0, 1000, 0, this));
	top.add("East", time_pos = new TextField(6));
	time_pos.setEditable(false);
	time_pos.setText("0");
	add("North", top);
	time_line.setTicks(500);

	noneSelected();
	setTitle("JShock");
	pack();
	show();
	}

	/**Load a presentation from a file, and make it the current one
	 */
	boolean loadFile(String f)
	{
	try {
		FileInputStream fis = new FileInputStream(f);
		Presentation p = new Presentation(fis);
		fis.close();
		file_name = f;
		edit_area.setPresentation(p);
		gotoTime(edit_area.pres.ctl.time);
		showTimeLines();
		setTitle("Jshock "+f+" ("+p.w+" x "+p.h+")");
		return true;
		}
	catch(Exception e) {
		return false;
		}
	}

	boolean saveFile(String f)
	{
	try {
		FileOutputStream fout = new FileOutputStream(file_name);
		edit_area.pres.save(fout);
		fout.close();
		setTitle("Jshock "+f+" ("+edit_area.pres.w+
			 " x "+edit_area.pres.h+")");
		return true;
		}
	catch(IOException e) {
		new ErrorWindow("Failed to save "+file_name+
		                " : "+e.getMessage());
		return false;
		}
	}

	/**Set the timeline slider and curret display to the given time
	 */
	void gotoTime(int t)
	{
	time_line.setPosition(t);
	time_pos.setText(String.valueOf(t));
	edit_area.pres.ctl.setTime(t);
	edit_area.repaint();
	for(int i=0; i<tlgoto_mitem.length; i++)
		tlgoto_mitem[i].setState(
			edit_area.pres.ctl == edit_area.pres.tl.elementAt(i));
	}

	/**Called by the edit area if no elements are selected
	 */
	void noneSelected()
	{
	cut_mitem.disable();
	copy_mitem.disable();
	del_mitem.disable();
	raise_mitem.disable();
	lower_mitem.disable();
	edit_mitem.disable();
	path_mitem.disable();
	}

	/**Called by the edit area if some items are selected
	 */
	void someSelected()
	{
	cut_mitem.enable();
	copy_mitem.enable();
	del_mitem.enable();
	raise_mitem.enable();
	lower_mitem.enable();
	edit_mitem.enable();
	path_mitem.enable();
	}

	/**Called by the edit area to setup the timelines menu
	 */
	void showTimeLines()
	{
	int c;
	while((c = timeline_menu.countItems()) > 4)
		timeline_menu.remove(c-1);
	Vector tl = edit_area.pres.tl;
	tlgoto_mitem = new CheckboxMenuItem[tl.size()];
	for(int i=0; i<tl.size(); i++)
		timeline_menu.add(tlgoto_mitem[i] =
		  new CheckboxMenuItem(((TimeLine)tl.elementAt(i)).name));
	int idx = edit_area.pres.tl.indexOf(edit_area.pres.ctl);
	if (idx != -1)
		tlgoto_mitem[idx].setState(true);
	}

	public void moved(CbSlider s, int p)
	{
	gotoTime(p);
	}

	public void moving(CbSlider s, int p)
	{
	gotoTime(p);
	}

	public boolean action(Event evt, Object obj)
	{
	Object t = evt.target;
	if (t == saveas_mitem) {
		FileDialog fd =
		  new FileDialog(this, "Save As..", FileDialog.SAVE);
		fd.show();
		if (fd.getFile() != null) {
			file_name = fd.getDirectory()+fd.getFile();
			saveFile(file_name);
			save_mitem.enable();
			}
		}
	else if (t == save_mitem) saveFile(file_name);
	else if (t == open_mitem) {
		// ask for a file to load
		FileDialog fd =
		  new FileDialog(this, "Open..", FileDialog.LOAD);
		fd.show();
		if (fd.getFile() != null) {
			file_name = fd.getDirectory()+fd.getFile();
			if (loadFile(file_name))
				save_mitem.enable();
			else
				new ErrorWindow("Failed to load "+file_name);
			}
		}
	else if (t == new_mitem) new PresentationSizeWindow();
	else if (t == close_mitem) dispose();
	else if (t == exit_mitem) System.exit(0);

	else if (t == cut_mitem) cutbuf = edit_area.cutSelected();
	else if (t == copy_mitem) cutbuf = edit_area.copySelected();
	else if (t == paste_mitem) edit_area.pasteElements(cutbuf);
	else if (t == del_mitem) edit_area.deleteSelected();
	else if (t == raise_mitem) edit_area.raiseSelected();
	else if (t == lower_mitem) edit_area.lowerSelected();
	else if (t == selall_mitem) edit_area.selectAll();
	else if (t == edit_mitem) edit_area.editSelected();
	else if (t == list_mitem) edit_area.openElementChooser();

	else if (t == path_mitem && pathwin == null) new PathWindow(this);
	else if (t == run_mitem) new RunWindow(edit_area.pres);
	else if (t == resize_mitem) new PresentationSizeWindow(this);
	else if (t == grid_mitem && gridwin == null) new GridWindow(this);
	else if (t == gridon_mitem) {
		gridon = !gridon;
		gridon_mitem.setState(gridon);
		if (gridon)
			edit_area.setGrid(gridw, gridh);
		else
			edit_area.setGrid(0, 0);
		}

	else if (t == tledit_mitem && tleditor == null)
		new TimeLineEditor(this, edit_area.pres.ctl);
	else if (t == tlactions_mitem)
		edit_area.editActions();
	else if (t == tlnew_mitem) new TimeLineEditor(this);
	else {
		for(int i=0; i<tlgoto_mitem.length; i++)
			if (t == tlgoto_mitem[i])
				edit_area.gotoTimeLine(i);
		}	
	return true;
	}

	public void dispose()
	{
	super.dispose();
	if (--wincount == 0)
		System.exit(0);
	}
}

class PresentationSizeWindow extends FixedFrame implements CbButtonCallback
{
	TextField width, height;
	CbButton ok, cancel;
	jshock parent;

	PresentationSizeWindow()
	{
	this(null);
	}

	PresentationSizeWindow(jshock p)
	{
	parent = p;
	setLayout(new BorderLayout());
	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(ok = new CbButton("Ok", this));
	bot.add(cancel = new CbButton("Cancel", this));
	add("South", bot);

	Panel mid = new BorderPanel(1);
	mid.setLayout(new GridLayout(2, 2, 2, 2));
	mid.add(new Label("Width:"));
	mid.add(width = new TextField(parent==null ?
		"500" : String.valueOf(parent.edit_area.pres.w)));
	mid.add(new Label("Height:"));
	mid.add(height = new TextField(parent==null ?
		"400" : String.valueOf(parent.edit_area.pres.h)));
	add("Center", mid);

	String t = parent==null ? "Create New Presentation" :
	                          "Resize Presentation";
	add("North", new Label(t));
	pack();
	show();
	setTitle(t);
	Util.recursiveBackground(this, Color.lightGray);
	}

	public void click(CbButton b)
	{
	if (b == ok) {
		int w, h;
		try { w = Integer.parseInt(width.getText()); }
		catch(Exception e) { w = -1; }
		try { h = Integer.parseInt(height.getText()); }
		catch(Exception e) { h = -1; }
		if (w <= 0) {
			new ErrorWindow("Width must be a positive integer");
			return;
			}
		if (h <= 0) {
			new ErrorWindow("Height must be a positive integer");
			return;
			}
		if (parent == null) new jshock(w, h);
		else {
			EditArea ea = parent.edit_area;
			ea.pres.w = w;
			ea.pres.h = h;
			ea.setPresentation(ea.pres);
			}
		}
	if (b == ok || b == cancel)
		dispose();
	}
}


class GridWindow extends FixedFrame implements CbButtonCallback
{
	jshock parent;
	TextField width, height;
	CbButton ok, cancel;

	GridWindow(jshock p)
	{
	parent = p;
	parent.gridwin = null;
	setLayout(new BorderLayout());

	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(ok = new CbButton("Ok", this));
	bot.add(cancel = new CbButton("Cancel", this));
	add("South", bot);

	Panel mid = new BorderPanel(1);
	mid.setLayout(new GridLayout(2, 2, 2, 2));
	mid.add(new Label("Width:"));
	mid.add(width = new TextField(String.valueOf(parent.gridw)));
	mid.add(new Label("Height:"));
	mid.add(height = new TextField(String.valueOf(parent.gridw)));
	add("Center", mid);

	add("North", new Label("Set Grid Size"));
	pack();
	show();
	setTitle("Set Grid Size");
	Util.recursiveBackground(this, Color.lightGray);
	}

	public void click(CbButton b)
	{
	if (b == ok) {
		int w, h;
		try { w = Integer.parseInt(width.getText()); }
		catch(Exception e) { w = -1; }
		try { h = Integer.parseInt(height.getText()); }
		catch(Exception e) { h = -1; }
		if (w <= 0) {
			new ErrorWindow("Width must be a positive integer");
			return;
			}
		if (h <= 0) {
			new ErrorWindow("Height must be a positive integer");
			return;
			}
		parent.gridw = w; parent.gridh = h;
		if (parent.gridon_mitem.getState())
			parent.edit_area.setGrid(w, h);
		}
	if (b == ok || b == cancel)
		dispose();
	}

	public void dispose()
	{
	parent.gridwin = null;
	super.dispose();
	}
}

class PathWindow extends FixedFrame implements CbButtonCallback
{
	jshock parent;
	EditArea ea;
	CbButton define, cancel;
	TextField start, end;
	Choice type;

	PathWindow(jshock p)
	{
	parent = p; ea = parent.edit_area;
	parent.pathwin = this;
	setLayout(new BorderLayout());
	Panel bot = new GrayPanel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(define = new CbButton("Define", this));
	bot.add(cancel = new CbButton("Cancel", this));
	add("South", bot);

	Panel mid = new BorderPanel(1);
	mid.setLayout(new BorderLayout());
	Panel left = new GrayPanel();
	left.setLayout(new GridLayout(0, 1));
	left.add(new Label("Start time:"));
	left.add(new Label("End time:"));
	left.add(new Label("Path type:"));
	mid.add("West", left);
	Panel right = new GrayPanel();
	right.setLayout(new GridLayout(0, 1));
	right.add(start = new TextField("0"));
	right.add(end = new TextField(
		String.valueOf(ea.pres.ctl.len)));
	right.add(type = new Choice());
	type.addItem("Polygon"); type.addItem("Freehand");
	mid.add("Center", right);
	add("Center", mid);
	add("North", new Label("Define movement path"));
	pack();
	show();
	setTitle("Define Path");
	Util.recursiveBackground(this, Color.lightGray);
	}

	public void click(CbButton b)
	{
	if (b == define) {
		int s, e;
		try { s = Integer.parseInt(start.getText()); }
		catch(Exception ex) { s = -1; }
		try { e = Integer.parseInt(end.getText()); }
		catch(Exception ex) { e = -1; }
		if (s < 0) {
			new ErrorWindow("Start must be a positive integer");
			return;
			}
		if (e < 0) {
			new ErrorWindow("End must be a positive integer");
			return;
			}
		if (e <= s) {
			new ErrorWindow("Start must be before end");
			return;
			}
		if (ea.selelm.size() == 0)
			new ErrorWindow("No elements selected");
		else {
			parent.gotoTime(s);
			ea.pathMode(s, e, type.getSelectedIndex()==0 ?
				EditArea.CREATE_POLY : EditArea.CREATE_FREE);
			}
		}
	if (b == define || b == cancel)
		dispose();
	}

	public void dispose()
	{
	parent.pathwin = null;
	super.dispose();
	}
}
